from utils import *
from gui import *

from . import Environment

class Car:
    def __init__(self, options: Options):
        self.options = options
        self.internal = Options({
            "position": 0, # meters
            "textures": [],
            "texturesLoaded": False
        })
        
    def __environment_init__(self, environment: Environment):
        options = self.options
        internal = self.internal
        self.environment = environment
        options.engine.__car_init__(self)
        if "cruiseControl" in options:
            options.cruiseControl.__car_init__(self)
        
    def loadAssets(self):
        options = self.options
        internal = self.internal
        internal.texturesLoaded = True
        files = options.files
        fitImage = options.fitImage
        segmentTotal = options.segmentTotal
        textures = internal.textures
        for fileName in files:
            print(f"Loading car file: {fileName}")
            himgui.load_asset_file_data(fileName)
            textureId = himgui.im_texture_id_from_asset(fileName)
            for y in range(0, fitImage):
                yStart = y / fitImage
                yEnd = (y + 1) / fitImage
                for x in range(0, fitImage):
                    xStart = x / fitImage
                    xEnd = (x + 1) / fitImage
                    textures.append(Options({
                        "file": fileName,
                        "textureId": textureId,
                        "xStart": xStart,
                        "yStart": yStart,
                        "xEnd": xEnd,
                        "yEnd": yEnd
                    }))
                    if len(textures) >= segmentTotal:
                        return
    
    def physicsLoop(self, timeDelta: float):
        options = self.options
        internal = self.internal
        
        options.engine.physicsLoop(timeDelta)
        if "cruiseControl" in options:
            options.cruiseControl.physicsLoop(timeDelta)
        internal.position += options.engine.internal.speed * timeDelta
    
    def drawLoop(self, timeDelta: float):
        options = self.options
        internal = self.internal
        if not internal.texturesLoaded:
            self.loadAssets()
        
        options.engine.drawLoop(timeDelta)
        if "cruiseControl" in options:
            options.cruiseControl.drawLoop(timeDelta)
        
        windowWidth = imgui.get_window_width()
        windowHeight = imgui.get_window_height()
        windowAspect = windowWidth / windowHeight
        aspect = options.aspect
        width = 0
        height = 0
        if aspect >= windowAspect:
            width = windowWidth
            height = windowWidth / aspect
        else:
            height = windowHeight
            width = windowHeight * aspect
        segmentGap = options.segmentGap
        position = internal.position
        textures = internal.textures
        currentImage = textures[math.floor(position / segmentGap) % len(textures)]
        # nextImage = textures[math.ceil(position / segmentGap) % len(textures)]
        # offsetImage = (position % segmentGap) / segmentGap
        
        imageSize = (width, height)
        currentUv0 = (currentImage.xStart, currentImage.yStart)
        currentUv1 = (currentImage.xEnd, currentImage.yEnd)
        # nextUv0 = (nextImage.xStart, nextImage.yStart)
        # nextUv1 = (nextImage.xEnd, nextImage.yEnd)
        
        cursorXBefore = imgui.get_cursor_pos_x()
        imgui.set_cursor_pos_x((internal.position - self.environment.options.scene.internal.position) / options.segmentLength * width)
        imgui.image(currentImage.textureId, imageSize, currentUv0, currentUv1, options.color)
        imgui.same_line()
        imgui.set_cursor_pos_x(cursorXBefore)
        # imgui.image(nextImage.textureId, imageSize, nextUv0, nextUv1, (1, 1, 1, offsetImage))
        # imgui.same_line()
        # imgui.set_cursor_pos_x(cursorXBefore)
    
    def metricsLoop(self, timeDelta: float):
        options = self.options
        internal = self.internal
        
        imgui.text(f"Position: {internal.position:.2f}m")
        options.engine.metricsLoop(timeDelta)
        if "cruiseControl" in options:
            options.cruiseControl.metricsLoop(timeDelta)
    
    def controlLoop(self, timeDelta: float):
        options = self.options
        internal = self.internal
        
        options.engine.controlLoop(timeDelta)
        if "cruiseControl" in options:
            options.cruiseControl.controlLoop(timeDelta)
        
        _, colorValue = imgui.color_edit4("Car Color", options.color, ColorEditFlags_.no_inputs | ColorEditFlags_.no_label)
        options.color = colorValue
