from utils import *
from gui import *

from . import Car

class CarCruiseControl:
    def __init__(self, options: Options):
        self.options = options
        self.internal = Options({
            "enabled": False,
            "lidarLastDistance": 0,
            "lidarRelativeSpeed": 0
        })
        self.metricsInternal = Options({
            "metrics": initHistory()
        })
    
    def __car_init__(self, car: Car):
        options = self.options
        internal = self.internal
        self.car = car
    
    def getLidarDistance(self):
        options = self.options
        internal = self.internal
        car = self.car
        environment = car.environment
        simulation = environment.simulation
        cars = simulation.internal.cars
        lidarDistance = math.inf # meters
        for otherCar in cars:
            if otherCar is car:
                continue
            distance = otherCar.internal.position - car.internal.position
            if distance < 0 or distance > 150:
                continue
            if lidarDistance > distance:
                lidarDistance = distance
        return lidarDistance
    
    def requestUserOverride(self):
        options = self.options
        internal = self.internal
        car = self.car
        internal.enabled = False
        car.options.engine.internal.throttle = 1
        car.options.engine.internal.brake = 0.5
    
    def onUserOverride(self):
        options = self.options
        internal = self.internal
        internal.enabled = False
    
    def physicsLoop(self, timeDelta: float):
        options = self.options
        internal = self.internal
        metricsInternal = self.metricsInternal
        metrics = metricsInternal.metrics
        car = self.car
        
        internal.safeDistance = car.options.engine.internal.speed * options.responseTime
        if not internal.enabled:
            metrics.pushHistory("Distance", internal.lidarLastDistance)
            metrics.pushHistory("Relative Speed", internal.lidarRelativeSpeed)
            metrics.pushHistory("Safe Distance", internal.safeDistance)
            return
        
        distance = self.getLidarDistance()
        if distance != math.inf:
            if internal.lidarLastDistance == 0:
                internal.lidarLastDistance = distance
            internal.lidarRelativeSpeed = internal.lidarRelativeSpeed * 0.6 + (distance - internal.lidarLastDistance) / timeDelta * 0.4
            internal.lidarLastDistance = distance
            if distance < internal.safeDistance:
                self.requestUserOverride()
            else:
                speedTarget = car.options.engine.internal.speed + internal.lidarRelativeSpeed
                targetTime = (distance - internal.safeDistance) / speedTarget
                targetAcceleration = 2 * (internal.safeDistance / targetTime ** 2 - car.options.engine.internal.speed)
                maximumBrakeAcceleration = car.options.engine.options.maximumBrakeForce / car.options.engine.options.mass
                brakeRatio = min(1, targetAcceleration / maximumBrakeAcceleration)
                car.options.engine.internal.brake = brakeRatio
        else:
            internal.lidarLastDistance = 0
        
        metrics.pushHistory("Distance", internal.lidarLastDistance)
        metrics.pushHistory("Relative Speed", internal.lidarRelativeSpeed)
        metrics.pushHistory("Safe Distance", internal.safeDistance)
    
    def drawLoop(self, timeDelta: float):
        pass
    
    def metricsLoop(self, timeDelta: float):
        options = self.options
        internal = self.internal
        metricsInternal = self.metricsInternal
        metrics = metricsInternal.metrics
        
        if imgui.begin_child("Cruise Control"):
            metrics.drawGraphTable()
        imgui.end_child()
    
    def controlLoop(self, timeDelta: float):
        options = self.options
        internal = self.internal
        if imgui.begin_child("Cruise Control"):
            _, enabledValue = imgui.checkbox("Enabled", internal.enabled)
            internal.enabled = enabledValue
        imgui.end_child()
