from utils import *
from gui import *

from . import CarEngine

class CarEngineTransmission():
    def __init__(self, options: Options):
        self.options = options
        self.internal = Options({})
    
    def __car_engine_init__(self, carEngine: CarEngine):
        options = self.options
        internal = self.internal
        self.carEngine = carEngine
    
    def physicsLoop(self, timeDelta: float):
        pass
    
    def drawLoop(self, timeDelta: float):
        pass
    
    def metricsLoop(self, timeDelta: float):
        pass
    
    def controlLoop(self, timeDelta: float):
        carEngine = self.carEngine
        if imgui.begin_child("Transmission"):
            canShiftUp = carEngine.testShift(carEngine.internal.gear + 1)
            if canShiftUp:
                imgui.push_style_color(imgui.Col_.button, ImVec4(0, 0.7, 0, 1))
                imgui.push_style_color(imgui.Col_.button_hovered, ImVec4(0.2, 0.7, 0.2, 1))
            if imgui.button("Shift Up"):
                carEngine.shiftUp()
            if canShiftUp:
                imgui.pop_style_color(2)
            
            canShiftDown = carEngine.testShift(carEngine.internal.gear - 1)
            if canShiftDown:
                imgui.push_style_color(imgui.Col_.button, ImVec4(0, 0.7, 0, 1))
                imgui.push_style_color(imgui.Col_.button_hovered, ImVec4(0.2, 0.7, 0.2, 1))
            if imgui.button("Shift Down"):
                carEngine.shiftDown()
            if canShiftDown:
                imgui.pop_style_color(2)
        imgui.end_child()

class CarEngineManualTransmission(CarEngineTransmission):
    def __init__(self, options: Options):
        super().__init__(options)

class CarEngineAutoTransmission(CarEngineTransmission):
    def __init__(self, options: Options):
        super().__init__(options)
    
    def physicsLoop(self, timeDelta: float):
        internal = self.internal
        internal.__current_loop_count__ = 0
        try:
            self.gearLoop(timeDelta)
        except Exception as e:
            if "Gear Bailout" in str(e):
                print(f"Gear bailout")
                return
            raise e
    
    def gearLoop(self, timeDelta: float):
        raise Exception("Not implemented")
    
    def __loop_protector__(self):
        internal = self.internal
        if internal.__current_loop_count__ >= 10:
            raise Exception("Gear Bailout")
        internal.__current_loop_count__ += 1
    
    def simulateWheelAcceleration(self, engineRpm: float, gearRatio: float, throttle: float):
        carEngine = self.carEngine
        carEngineOptions = carEngine.options
        carEngineInternal = carEngine.internal
        engineTorque = CarEngine.getEngineTorque0(engineRpm, throttle, carEngineOptions.maxTorque, carEngineOptions.maxTorqueRpm)
        wheelTorque = CarEngine.getWheelTorque0(engineTorque, gearRatio, carEngineOptions.finalDrive)
        wheelForce = wheelTorque / (carEngineOptions.tireDiameter / 2)
        wheelAcceleration = wheelForce / carEngineOptions.mass
        return wheelAcceleration
    
    def simulateEngineRpm(self, wheelSpeed: float, gearRatio: float):
        carEngine = self.carEngine
        carEngineOptions = carEngine.options
        carEngineInternal = carEngine.internal
        engineRpm = CarEngine.getRpmFromSpeed(wheelSpeed, gearRatio, carEngineOptions.finalDrive, carEngineOptions.tireDiameter)
        return engineRpm

class CarEngineAutoTransmissionR3(CarEngineAutoTransmission):
    def __init__(self, options: Options):
        super().__init__(options)
    
    def gearLoop(self, timeDelta: float):
        carEngine = self.carEngine
        carEngineOptions = carEngine.options
        carEngineInternal = carEngine.internal
        if carEngineInternal.shiftProgress < 1:
            return
        while True:
            self.__loop_protector__()
            maxSensibleRpm = 0
            if carEngineInternal.gear < len(carEngineOptions.gearRatios) - 1:
                currentGearRatio = carEngineOptions.gearRatios[carEngineInternal.gear]
                nextGearRatio = carEngineOptions.gearRatios[carEngineInternal.gear + 1]
                maxSensibleRpm = 2 * currentGearRatio * carEngineOptions.maxTorqueRpm * (nextGearRatio ** 2 - currentGearRatio ** 2) / (nextGearRatio ** 3 - currentGearRatio ** 3)
            else:
                lastGearRatio = carEngineOptions.gearRatios[len(carEngineOptions.gearRatios) - 1]
                secondLastGearRatio = carEngineOptions.gearRatios[len(carEngineOptions.gearRatios) - 2]
                maxSensibleRpm = 2 * secondLastGearRatio * carEngineOptions.maxTorqueRpm * (lastGearRatio ** 2 - secondLastGearRatio ** 2) / (lastGearRatio ** 3 - secondLastGearRatio ** 3)
            if maxSensibleRpm > carEngineOptions.rpmLimit:
                maxSensibleRpm = carEngineOptions.rpmLimit - 1
            targetRpm = (1 - carEngineInternal.throttle ** 3) * (carEngineOptions.rpmIdle) + (carEngineInternal.throttle ** 3) * (maxSensibleRpm)
            currentTargetRpm = self.simulateEngineRpm(carEngineInternal.speed, carEngineOptions.gearRatios[carEngineInternal.gear])
            if currentTargetRpm >= maxSensibleRpm:
                carEngine.shiftUp()
            elif currentTargetRpm < carEngineOptions.rpmIdle:
                carEngine.shiftDown()
            bestGear = carEngineInternal.gear
            bestDifference = abs(targetRpm - currentTargetRpm)
            gearHuntThreshold = (carEngineOptions.rpmLimit - carEngineOptions.rpmIdle) / 6
            if bestDifference > gearHuntThreshold:
                for gear in range(0, len(carEngineOptions.gearRatios)):
                    gearRatio = carEngineOptions.gearRatios[gear]
                    currentGearRatio = carEngineOptions.gearRatios[carEngineInternal.gear]
                    testRpm = currentTargetRpm * (gearRatio / currentGearRatio)
                    testDifference = abs(targetRpm - testRpm)
                    if testDifference >= bestDifference:
                        continue
                    bestGear = gear
                    bestDifference = testDifference
            testRpm = self.simulateEngineRpm(carEngineInternal.speed, carEngineOptions.gearRatios[bestGear])
            if carEngineInternal.gear < bestGear and testRpm > carEngineOptions.rpmIdle:
                carEngine.shiftUp()
                continue
            elif carEngineInternal.gear > bestGear and testRpm <= maxSensibleRpm:
                carEngine.shiftDown()
                continue
            break

class CarEngineAutoTransmissionAcceleration(CarEngineAutoTransmission):
    def __init__(self, options: Options):
        super().__init__(options)
    
    def gearLoop(self, timeDelta: float):
        carEngine = self.carEngine
        carEngineOptions = carEngine.options
        carEngineInternal = carEngine.internal
        if carEngineInternal.shiftProgress < 1:
            return
        while True:
            self.__loop_protector__()
            optimalGearRatio = (carEngineOptions.rpmMaxPower * math.pi * carEngineOptions.tireDiameter) / (60 * max(0.001, carEngineInternal.speed) * carEngineOptions.finalDrive)
            if optimalGearRatio < carEngineOptions.gearRatios[len(carEngineOptions.gearRatios) - 1]:
                optimalGearRatio = carEngineOptions.gearRatios[len(carEngineOptions.gearRatios) - 1]
            elif optimalGearRatio > carEngineOptions.gearRatios[0]:
                optimalGearRatio = carEngineOptions.gearRatios[0]
            maximumAcceleration = self.simulateWheelAcceleration(carEngineOptions.rpmMaxPower, optimalGearRatio, 1)
            targetAcceleration = maximumAcceleration * carEngineInternal.throttle ** 3
            bestGear = carEngineInternal.gear
            bestDifference = abs(targetAcceleration - carEngine.getWheelAcceleration())
            for gear in range(0, len(carEngineOptions.gearRatios)):
                gearRatio = carEngineOptions.gearRatios[gear]
                testRpm = CarEngine.getRpmFromSpeed(carEngineInternal.speed, gearRatio, carEngineOptions.finalDrive, carEngineOptions.tireDiameter)
                testDifference = abs(targetAcceleration - self.simulateWheelAcceleration(testRpm, gearRatio, carEngineInternal.throttle))
                if testDifference >= bestDifference:
                    continue
                bestGear = gear
                bestDifference = testDifference
            checkNext = False
            rpmCheck = CarEngine.getRpmFromSpeed(carEngineInternal.speed, carEngineOptions.gearRatios[bestGear], carEngineOptions.finalDrive, carEngineOptions.tireDiameter)
            if carEngineInternal.gear < bestGear and rpmCheck > carEngineOptions.rpmIdle:
                carEngine.shiftUp()
                checkNext = True
            elif carEngineInternal.gear > bestGear and rpmCheck < carEngineOptions.rpmLimit:
                carEngine.shiftDown()
                checkNext = True
            if carEngineInternal.rpm >= carEngineOptions.rpmLimit - 1:
                carEngine.shiftUp()
            elif carEngineInternal.rpm < carEngineOptions.rpmIdle:
                carEngine.shiftDown()
            if not checkNext:
                break
