from utils import *
from gui import *

class Environment:
    def __init__(self, options):
        self.options = Options({
            "scene": options.scene,
            "airDensity": 1.225,
            "airDensityVariance": 0.12,
            "windVelocity": 9.25,
            "windVelocityVariance": 0.25,
        })
        self.internal = Options({
            "airDensity": 0,
            "windVelocity": 0
        })
        self.metricsInternal = Options({
            "metrics": initHistory()
        })
        
    def __init_simulation__(self, simulation):
        options = self.options
        internal = self.internal
        self.simulation = simulation
        self.options.scene.__environment_init__(self)
    
    def getWindPressure(self):
        options = self.options
        internal = self.internal
        windPressure = 0.5 * internal.airDensity * internal.windVelocity ** 2
        return windPressure
    
    def getWindForce(self, area, angle=math.pi/2):
        windForce = self.getWindPressure() * area * math.sin(angle)
        return windForce

    def physicsLoop(self, timeDelta: float):
        options = self.options
        internal = self.internal
        metricsInternal = self.metricsInternal
        metrics = metricsInternal.metrics
        
        internal.airDensity = options.airDensity + (random.random() * 2 - 1) * options.airDensityVariance
        internal.windVelocity = options.windVelocity + (random.random() * 2 - 1) * options.windVelocityVariance
        
        metrics.pushHistory("Air Density", internal.airDensity) # kg/m^3
        metrics.pushHistory("Wind Velocity", internal.windVelocity) # m/s
        metrics.pushHistory("Wind Pressure", self.getWindPressure()) # Pa
    
    def drawLoop(self, timeDelta: float):
        options = self.options
        internal = self.internal
        
        options.scene.drawLoop(timeDelta)
    
    def metricsLoop(self, timeDelta: float):
        options = self.options
        internal = self.internal
        metricsInternal = self.metricsInternal
        metrics = metricsInternal.metrics
        
        if imgui.begin_child("Environment"):
            metrics.drawGraphTable()
        imgui.end_child()
    
    def controlLoop(self, timeDelta: float):
        options = self.options
        internal = self.internal
        
        if imgui.begin_child("Environment"):
            _, airDensityValue = imgui.slider_float("Air Density", options.airDensity, v_min=0, v_max=5, format="%.2f")
            options.airDensity = airDensityValue
            
            _, airDensityVarianceValue = imgui.slider_float("Air Density Variance", options.airDensityVariance, v_min=0, v_max=0.25, format="%.4f")
            options.airDensityVariance = airDensityVarianceValue
            
            _, windVelocityValue = imgui.slider_float("Wind Velocity", options.windVelocity, v_min=0, v_max=20, format="%.2f")
            options.windVelocity = windVelocityValue
            
            _, windVelocityVarianceValue = imgui.slider_float("Wind Velocity Variance", options.windVelocityVariance, v_min=0, v_max=0.5, format="%.4f")
            options.windVelocityVariance = windVelocityVarianceValue
            
        imgui.end_child()
