from utils import *
from gui import *

from . import Environment

class Scene:
    def __init__(self, options):
        self.options = options
        self.internal = Options({
            "position": 0, # meters
            "textures": [],
            "texturesLoaded": False
        })
        
    def __environment_init__(self, environment: Environment):
        options = self.options
        internal = self.internal
        self.environment = environment
        
    def loadAssets(self):
        options = self.options
        internal = self.internal
        internal.texturesLoaded = True
        files = options.files
        fitImage = options.fitImage
        segmentTotal = options.segmentTotal
        textures = internal.textures
        for fileName in files:
            print(f"Loading scene file: {fileName}")
            himgui.load_asset_file_data(fileName)
            textureId = himgui.im_texture_id_from_asset(fileName)
            for y in range(0, fitImage):
                yStart = y / fitImage
                yEnd = (y + 1) / fitImage
                for x in range(0, fitImage):
                    xStart = x / fitImage
                    xEnd = (x + 1) / fitImage
                    textures.append(Options({
                        "file": fileName,
                        "textureId": textureId,
                        "xStart": xStart,
                        "yStart": yStart,
                        "xEnd": xEnd,
                        "yEnd": yEnd
                    }))
                    if len(textures) >= segmentTotal:
                        return
        
    def drawLoop(self, timeDelta: float):
        options = self.options
        internal = self.internal
        if not internal.texturesLoaded:
            self.loadAssets()
        
        windowWidth = imgui.get_window_width()
        windowHeight = imgui.get_window_height()
        windowAspect = windowWidth / windowHeight
        aspect = options.aspect
        width = 0
        height = 0
        if aspect >= windowAspect:
            width = windowWidth
            height = windowWidth / aspect
        else:
            height = windowHeight
            width = windowHeight * aspect
        fitImage = options.fitImage
        segmentGap = options.segmentGap
        segmentGapUv = options.segmentGapUv / fitImage
        segmentLength = options.segmentGapUv * width
        position = internal.position
        textures = internal.textures
        currentImage = textures[math.floor(position / segmentGap) % len(textures)]
        nextImage = textures[math.ceil(position / segmentGap) % len(textures)]
        offsetImage = (position % segmentGap) / segmentGap
        
        currentSize = (map(offsetImage, 0, 1, segmentLength, 0), height)
        currentUv0 = (map(offsetImage, 0, 1, currentImage.xStart, currentImage.xStart + segmentGapUv), currentImage.yStart)
        currentUv1 = (currentImage.xStart + segmentGapUv, currentImage.yEnd)
        currentMixSize = (width - segmentLength, height)
        currentMixUv0 = (currentImage.xStart + segmentGapUv, currentImage.yStart)
        currentMixUv1 = (currentImage.xEnd, currentImage.yEnd)
        nextMixSize = (width - segmentLength, height)
        nextMixUv0 = (nextImage.xStart, nextImage.yStart)
        nextMixUv1 = (nextImage.xEnd - segmentGapUv, nextImage.yEnd)
        nextSize = (map(offsetImage, 0, 1, 0, segmentLength), height)
        nextUv0 = (nextImage.xEnd - segmentGapUv, nextImage.yStart)
        nextUv1 = (map(offsetImage, 0, 1, nextImage.xEnd - segmentGapUv, nextImage.xEnd), nextImage.yEnd)
        
        cursorXBefore = imgui.get_cursor_pos_x()
        imgui.image(currentImage.textureId, currentSize, currentUv0, currentUv1)
        imgui.same_line()
        cursorXAnchor = imgui.get_cursor_pos_x() - 8
        imgui.set_cursor_pos_x(cursorXAnchor)
        imgui.image(currentImage.textureId, currentMixSize, currentMixUv0, currentMixUv1)
        imgui.same_line()
        imgui.set_cursor_pos_x(cursorXAnchor)
        imgui.image(nextImage.textureId, nextMixSize, nextMixUv0, nextMixUv1, (1, 1, 1, offsetImage))
        imgui.same_line()
        imgui.set_cursor_pos_x(imgui.get_cursor_pos_x() - 8)
        imgui.image(nextImage.textureId, nextSize, nextUv0, nextUv1)
        imgui.same_line()
        imgui.set_cursor_pos_x(cursorXBefore)
