from .imgui import *
from utils import *

def initHistory():
    metrics = Options({})
    def pushHistory(property: str, value: float):
        key = f"__{property}"
        if key not in metrics:
            metrics[key] = Options({
                "name": property,
                "skip": 0,
                "value": 0,
                "index": 0,
                "min": 0,
                "max": 0,
                "history": np.zeros(500)
            })
        metric = metrics[key]
        metric.skip += 1
        if metric.skip < 2:
            return
        metric.skip = 0
        metric.value = value
        metric.history[metric.index] = value
        metric.index += 1
        metric.min = 0.7 * metric.min + 0.3 * min(metric.history)
        metric.max = 0.7 * metric.max + 0.3 * max(metric.history)
        if metric.index >= len(metric.history):
            metric.index = 0
    def drawGraph(property: str, color: ImVec4):
        key = f"__{property}"
        metric = metrics[key]
        implot.push_style_var(implot.StyleVar_.plot_padding, ImVec2(0, 0))
        if implot.begin_plot(metric.name, ImVec2(-1, 35), ImplotFlags_.canvas_only):
            implot.setup_axes(None, None, AxisFlags_.no_decorations, AxisFlags_.no_decorations)
            implot.setup_axes_limits(0, len(metric.history), metric.min - 0.01, metric.max + 0.01, Cond_.always)
            implot.set_next_line_style(color)
            implot.set_next_fill_style(color, 0.25)
            implot.plot_line(f"{metric.name} End", metric.history[metric.index:len(metric.history)], 1, 0, LineFlags_.shaded)
            implot.set_next_line_style(color)
            implot.set_next_fill_style(color, 0.25)
            implot.plot_line(f"{metric.name} Start", metric.history[0:metric.index], 1, (1 - (metric.index + 1) / len(metric.history)) * len(metric.history), LineFlags_.shaded)
            implot.end_plot()
        implot.pop_style_var()
    def drawGraphTable():
        if imgui.begin_table("Graphs", 3, TableFlags_.borders_outer | TableFlags_.borders_v | TableFlags_.row_bg | TableFlags_.resizable | TableFlags_.reorderable, ImVec2(-1, 0)):
            imgui.table_setup_column("Property", TableColumnFlags_.width_fixed, 75)
            imgui.table_setup_column("Value", TableColumnFlags_.width_fixed, 75)
            imgui.table_setup_column("History")
            imgui.table_headers_row()
            for key, metric in metrics.items():
                if not key.startswith("__"):
                    continue
                imgui.table_next_row()
                imgui.table_set_column_index(0)
                imgui.text(metric.name)
                imgui.table_set_column_index(1)
                imgui.text(f"{metric.value}")
                imgui.table_set_column_index(2)
                imgui.push_id(metric.name)
                drawGraph(metric.name, implot.get_colormap_color(abs(hash(metric.name)) % 10))
                imgui.pop_id()
            imgui.end_table()
    metrics.pushHistory = pushHistory
    metrics.drawGraph = drawGraph
    metrics.drawGraphTable = drawGraphTable
    return metrics
